"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
// third-party
const react_1 = __importDefault(require("react"));
// components
const SuccessIcon_1 = __importDefault(require("./SuccessIcon"));
const ErrorIcon_1 = __importDefault(require("./ErrorIcon"));
const InfoIcon_1 = __importDefault(require("./InfoIcon"));
const WarningIcon_1 = __importDefault(require("./WarningIcon"));
const CustomIcon_1 = __importDefault(require("./CustomIcon"));
const Buttons_1 = __importDefault(require("./Buttons"));
const Input_1 = __importDefault(require("./Input"));
const ValidationMessage_1 = __importDefault(require("./ValidationMessage"));
const Title_1 = __importDefault(require("./Title"));
const Content_1 = __importDefault(require("./Content"));
const Overlay_1 = __importDefault(require("./Overlay"));
// other
const styles = __importStar(require("../styles/SweetAlertStyles"));
const Patterns = __importStar(require("../constants/patterns"));
const prop_types_1 = require("../prop-types");
const default_props_1 = require("../default-props");
const SWEET_ALERT_DEFAULT_STYLES = styles.sweetAlert;
const _resetting = {};
const debugLogger = (...args) => {
    // uncomment the next line to get some debugging logs.
    // console.log(...args);
};
class SweetAlert extends react_1.default.Component {
    constructor(props) {
        super(props);
        this.inputElement = null;
        this.unsupportedProp = (oldProp, message) => {
            try {
                console.warn(`react-bootstrap-sweetalert: Unsupported prop '${oldProp}'. Please ${message}`);
            }
            catch (e) {
                // do nothing
            }
        };
        this.focusInput = () => {
            debugLogger('inputElement', this.inputElement);
            if (this.inputElement) {
                debugLogger('inputElement trying to focus', this.inputElement);
                try {
                    this.inputElement.focus();
                }
                catch (e) {
                    debugLogger('inputElement focus error', e);
                    // whoops
                }
            }
        };
        this.getIcon = () => {
            switch (this.state.type) {
                case 'danger':
                case 'error':
                    return react_1.default.createElement(ErrorIcon_1.default, null);
                case 'warning':
                    return react_1.default.createElement(WarningIcon_1.default, null);
                case 'info':
                    return react_1.default.createElement(InfoIcon_1.default, null);
                case 'success':
                    return react_1.default.createElement(SuccessIcon_1.default, null);
                case 'custom':
                    if (this.props.customIcon) {
                        if (typeof this.props.customIcon == 'string') {
                            return react_1.default.createElement(CustomIcon_1.default, { iconUrl: this.props.customIcon });
                        }
                        return this.props.customIcon;
                    }
                    return null;
                default:
                    return null;
            }
        };
        this.onChangeInput = (e) => {
            const target = e.target;
            this.setState({
                inputValue: target.value,
                showValidationMessage: false
            });
        };
        this.isValidInput = () => {
            if (!this.props.required) {
                return true;
            }
            const regex = this.props.validationRegex || (this.props.inputType === 'email' ? Patterns.emailRegex : Patterns.defaultRegex);
            return regex.test(this.state.inputValue);
        };
        this.isDisabled = () => {
            return this.props.onCancel && this.props.disabled;
        };
        this.onAlertClose = (callback) => {
            _resetting[this.state.id] = true;
            debugLogger('onAlertClose resetting state');
            this.setState({
                ...SweetAlert.getDefaultState(),
                id: this.state.id,
            }, () => {
                _resetting[this.state.id] = false;
                callback();
            });
        };
        this.beforeCloseAlert = (closingAction, callback) => {
            debugLogger('in beforeCloseAlert: setting show to false');
            this.setState({ show: false, closingAction }, () => {
                debugLogger('state updated', this.state.show);
                if (!this.state.show) {
                    debugLogger('invoking callback');
                    callback();
                }
            });
        };
        this.onConfirm = (handleCloseAnimations = true) => {
            if (this.isDisabled()) {
                return;
            }
            // if this is an input alert, then we will send the input value to the props.onConfirm function
            const isInput = this.state.type === 'input';
            const inputValue = this.state.inputValue;
            // if this is a controlled alert, then we will send the dependencies value to the props.onConfirm function
            const isControlled = this.state.type === 'controlled';
            const dependencies = [...this.state.dependencies];
            if (isInput && !this.isValidInput()) {
                this.setState({
                    showValidationMessage: true
                });
                return;
            }
            const confirm = () => {
                debugLogger('in confirm callback');
                if (isInput) {
                    this.onAlertClose(() => {
                        this.props.onConfirm(inputValue);
                    });
                }
                else if (isControlled) {
                    this.onAlertClose(() => {
                        this.props.onConfirm(dependencies);
                    });
                }
                else {
                    this.onAlertClose(() => this.props.onConfirm());
                }
            };
            if (handleCloseAnimations) {
                debugLogger('calling beforeCloseAlert');
                this.beforeCloseAlert('confirm', () => confirm());
            }
            else {
                confirm();
            }
        };
        this.onCancel = (handleCloseAnimations = true) => {
            const cancel = () => {
                this.onAlertClose(() => {
                    if (this.props.onCancel) {
                        this.props.onCancel();
                    }
                });
            };
            if (handleCloseAnimations) {
                this.beforeCloseAlert('cancel', () => cancel());
            }
            else {
                cancel();
            }
        };
        this.onInputKeyDown = (e) => {
            if (e.keyCode == 13) {
                e.stopPropagation();
                this.onConfirm();
            }
        };
        this.onKeyDown = (e) => {
            if (e.keyCode == 27) {
                if (this.props.allowEscape && this.props.onCancel) {
                    e.stopPropagation();
                    this.onCancel();
                }
            }
        };
        this.onClickInside = (e) => {
            e.stopPropagation();
        };
        this.onClickOutside = () => {
            if (this.props.closeOnClickOutside && this.props.onCancel) {
                this.onCancel();
            }
        };
        this.setAutoFocusInputRef = (element) => {
            this.inputElement = element;
        };
        this.getComposedStyle = () => {
            return Object.assign({}, SWEET_ALERT_DEFAULT_STYLES, this.props.style, { animation: this.state.animation });
        };
        this.getAlertContent = () => {
            // Support for render props for content of alert
            if (typeof this.props.children === 'function') {
                const renderProps = {
                    onEnterKeyDownConfirm: (event) => {
                        if (event.key === 'Enter') {
                            event.preventDefault();
                            event.stopPropagation();
                            this.onConfirm();
                        }
                    },
                    confirm: () => this.onConfirm(),
                    cancel: () => this.onCancel(),
                    setAutoFocusInputRef: this.setAutoFocusInputRef.bind(this),
                };
                return this.props.children(renderProps);
            }
            return this.props.children;
        };
        this.getCloseButton = () => {
            if (!this.props.showCloseButton || !this.props.onCancel) {
                return null;
            }
            return (react_1.default.createElement("span", { className: 'btn', style: Object.assign({}, styles.closeButton, this.props.closeBtnStyle), onClick: () => this.onCancel() }, "x"));
        };
        this.getInputField = () => {
            if (this.state.type !== 'input') {
                return null;
            }
            return (react_1.default.createElement(Input_1.default, Object.assign({}, this.props, this.state, { type: this.state.type, onInputKeyDown: this.onInputKeyDown, onChangeInput: this.onChangeInput })));
        };
        this.getValidationMessage = () => {
            if (!this.state.showValidationMessage) {
                return null;
            }
            return react_1.default.createElement(ValidationMessage_1.default, Object.assign({}, this.props));
        };
        this.getButtons = () => {
            return (react_1.default.createElement(Buttons_1.default, Object.assign({}, this.props, { type: this.state.type, onConfirm: this.onConfirm, onCancel: this.onCancel, focusConfirmBtn: this.state.focusConfirmBtn, focusCancelBtn: this.state.focusCancelBtn, disabled: this.isDisabled() })));
        };
        this.getInjectedStyles = () => {
            return (react_1.default.createElement(react_1.default.Fragment, null,
                react_1.default.createElement("style", { type: "text/css", dangerouslySetInnerHTML: {
                        __html: `
                body.sweetalert-overflow-hidden {
                  overflow: hidden;
                }
                body .sweet-alert button {
                  outline: none !important;
                }
              `
                    } }),
                react_1.default.createElement("style", { type: "text/css" }, `@-webkit-keyframes showSweetAlert {  0% {    transform: scale(0.7);    -webkit-transform: scale(0.7);  }  45% {    transform: scale(1.05);    -webkit-transform: scale(1.05);  }  80% {    transform: scale(0.95);    -webkit-tranform: scale(0.95);  }  100% {    transform: scale(1);    -webkit-transform: scale(1);  }}@keyframes showSweetAlert {  0% {    transform: scale(0.7);    -webkit-transform: scale(0.7);  }  45% {    transform: scale(1.05);    -webkit-transform: scale(1.05);  }  80% {    transform: scale(0.95);    -webkit-tranform: scale(0.95);  }  100% {    transform: scale(1);    -webkit-transform: scale(1);  }}@-webkit-keyframes hideSweetAlert {  0% {    transform: scale(1);    -webkit-transform: scale(1);  }  100% {    transform: scale(0.4);    -webkit-transform: scale(0.4);  }}@keyframes hideSweetAlert {  0% {    transform: scale(1);    -webkit-transform: scale(1);  }  100% {    transform: scale(0.4);    -webkit-transform: scale(0.4);  }}@-webkit-keyframes animateSuccessTip {  0% {    width: 0;    left: 1px;    top: 19px;  }  54% {    width: 0;    left: 1px;    top: 19px;  }  70% {    width: 50px;    left: -8px;    top: 37px;  }  84% {    width: 17px;    left: 21px;    top: 48px;  }  100% {    width: 25px;    left: 14px;    top: 45px;  }}@keyframes animateSuccessTip {  0% {    width: 0;    left: 1px;    top: 19px;  }  54% {    width: 0;    left: 1px;    top: 19px;  }  70% {    width: 50px;    left: -8px;    top: 37px;  }  84% {    width: 17px;    left: 21px;    top: 48px;  }  100% {    width: 25px;    left: 14px;    top: 45px;  }}@-webkit-keyframes animateSuccessLong {  0% {    width: 0;    right: 46px;    top: 54px;  }  65% {    width: 0;    right: 46px;    top: 54px;  }  84% {    width: 55px;    right: 0px;    top: 35px;  }  100% {    width: 47px;    right: 8px;    top: 38px;  }}@keyframes animateSuccessLong {  0% {    width: 0;    right: 46px;    top: 54px;  }  65% {    width: 0;    right: 46px;    top: 54px;  }  84% {    width: 55px;    right: 0px;    top: 35px;  }  100% {    width: 47px;    right: 8px;    top: 38px;  }}@-webkit-keyframes rotatePlaceholder {  0% {    transform: rotate(-45deg);    -webkit-transform: rotate(-45deg);  }  5% {    transform: rotate(-45deg);    -webkit-transform: rotate(-45deg);  }  12% {    transform: rotate(-405deg);    -webkit-transform: rotate(-405deg);  }  100% {    transform: rotate(-405deg);    -webkit-transform: rotate(-405deg);  }}@keyframes rotatePlaceholder {  0% {    transform: rotate(-45deg);    -webkit-transform: rotate(-45deg);  }  5% {    transform: rotate(-45deg);    -webkit-transform: rotate(-45deg);  }  12% {    transform: rotate(-405deg);    -webkit-transform: rotate(-405deg);  }  100% {    transform: rotate(-405deg);    -webkit-transform: rotate(-405deg);  }}@-webkit-keyframes animateErrorIcon {  0% {    transform: rotateX(100deg);    -webkit-transform: rotateX(100deg);    opacity: 0;  }  100% {    transform: rotateX(0deg);    -webkit-transform: rotateX(0deg);    opacity: 1;  }}@keyframes animateErrorIcon {  0% {    transform: rotateX(100deg);    -webkit-transform: rotateX(100deg);    opacity: 0;  }  100% {    transform: rotateX(0deg);    -webkit-transform: rotateX(0deg);    opacity: 1;  }}@-webkit-keyframes animateXMark {  0% {    transform: scale(0.4);    -webkit-transform: scale(0.4);    margin-top: 26px;    opacity: 0;  }  50% {    transform: scale(0.4);    -webkit-transform: scale(0.4);    margin-top: 26px;    opacity: 0;  }  80% {    transform: scale(1.15);    -webkit-transform: scale(1.15);    margin-top: -6px;  }  100% {    transform: scale(1);    -webkit-transform: scale(1);    margin-top: 0;    opacity: 1;  }}@keyframes animateXMark {  0% {    transform: scale(0.4);    -webkit-transform: scale(0.4);    margin-top: 26px;    opacity: 0;  }  50% {    transform: scale(0.4);    -webkit-transform: scale(0.4);    margin-top: 26px;    opacity: 0;  }  80% {    transform: scale(1.15);    -webkit-transform: scale(1.15);    margin-top: -6px;  }  100% {    transform: scale(1);    -webkit-transform: scale(1);    margin-top: 0;    opacity: 1;  }}@-webkit-keyframes pulseWarning {  0% {    border-color: #F8D486;  }  100% {    border-color: #F8BB86;  }}@keyframes pulseWarning {  0% {    border-color: #F8D486;  }  100% {    border-color: #F8BB86;  }}@-webkit-keyframes pulseWarningIns {  0% {    background-color: #F8D486;  }  100% {    background-color: #F8BB86;  }}@keyframes pulseWarningIns {  0% {    background-color: #F8D486;  }  100% {    background-color: #F8BB86;  }}`)));
        };
        if (this.props.beforeUpdate) {
            this.unsupportedProp('beforeUpdate', 'use props.afterUpdate');
        }
        const newState = Object.assign(SweetAlert.getDefaultState(), {
            hideTimeoutHandlerFunc: this.hideTimeoutHandler.bind(this)
        });
        if (this.props.defaultValue != null) {
            newState.inputValue = this.props.defaultValue;
        }
        this.state = newState;
        this.props.beforeMount();
    }
    componentDidMount() {
        document.body.classList.add('sweetalert-overflow-hidden');
        this.focusInput();
        if (this.props.afterMount) {
            this.props.afterMount();
        }
    }
    static generateId() {
        return '' + Date.now() + Math.ceil(Math.random() * 10000000000) + Math.ceil(Math.random() * 10000000000);
    }
    static getDefaultState() {
        return {
            id: SweetAlert.generateId(),
            show: true,
            focusConfirmBtn: true,
            focusCancelBtn: false,
            inputValue: '',
            showValidationMessage: false,
            timer: null,
            animation: "",
            prevTimeout: 0,
            closingAction: null,
            dependencies: [],
        };
    }
    static getDerivedStateFromProps(nextProps, nextState) {
        if (_resetting[nextState.id]) {
            return {};
        }
        let newState = {};
        const typeChanged = nextState.type !== SweetAlert.getTypeFromProps(nextProps);
        const dependenciesChanged = nextState.dependencies !== nextProps.dependencies;
        const timeoutChanged = nextState.prevTimeout !== nextProps.timeout;
        // if the type of the alert changed, or the dependencies changed, then update the state from props
        if (typeChanged || dependenciesChanged) {
            newState = {
                ...newState,
                ...SweetAlert.getStateFromProps(nextProps),
            };
        }
        // if the state is changing, or the timeout changed, then reset the timeout timer
        if (JSON.stringify(newState) !== '{}' || timeoutChanged) {
            newState = {
                ...newState,
                ...SweetAlert.handleTimeout(nextProps, nextState.timer)
            };
        }
        // return the partially updated state
        return {
            ...newState,
            ...SweetAlert.handleAnimState(nextProps, nextState, nextState.hideTimeoutHandlerFunc),
        };
    }
    componentDidUpdate(prevProps, prevState) {
        if (this.props.beforeUpdate) {
            this.props.beforeUpdate(prevProps, prevState);
        }
        if (!prevState.show && this.state.show) {
            this.focusInput();
        }
        this.props.afterUpdate(this.props, this.state);
    }
    componentWillUnmount() {
        document.body.classList.remove('sweetalert-overflow-hidden');
        if (this.state.timer) {
            clearTimeout(this.state.timer);
        }
        if (this.props.beforeUnmount) {
            this.props.beforeUnmount();
        }
    }
    hideTimeoutHandler(time) {
        setTimeout(() => {
            const closingAction = this.state.closingAction;
            /**
             * Removing the closing action (shouldn't trigger another animation timeout)
             */
            this.setState({ show: false, closingAction: null }, () => {
                // handle the action that was started before the closing animation was started
                switch (closingAction) {
                    case 'confirm':
                        this.onConfirm(false);
                        break;
                    case 'cancel':
                        this.onCancel(false);
                        break;
                    default:
                        break;
                }
            });
        }, time);
    }
    ;
    static handleTimeout(props, currentTimer) {
        if (currentTimer) {
            clearTimeout(currentTimer);
        }
        if (props.timeout && props.timeout > 0) {
            const timer = setTimeout(() => props.onConfirm(), props.timeout);
            return { timer: timer, prevTimeout: props.timeout };
        }
        return null;
    }
    static isAnimation(animProp) {
        return animProp && typeof animProp !== 'boolean';
    }
    static animationFromProp(animProp) {
        return animProp.name + ' ' + animProp.duration + 'ms';
    }
    static handleAnimState(props, state, hideTimeout) {
        const userDefinedShow = typeof props.show === 'boolean';
        let show = (userDefinedShow && !state.closingAction) ? props.show : state.show;
        let animation = '';
        if (show) {
            if (props.openAnim) {
                if (SweetAlert.isAnimation(props.openAnim)) {
                    animation = SweetAlert.animationFromProp(props.openAnim);
                }
                else if (SweetAlert.isAnimation(SweetAlert.defaultProps.openAnim)) {
                    animation = SweetAlert.animationFromProp(SweetAlert.defaultProps.openAnim);
                }
            }
        }
        else if (state.closingAction && props.closeAnim) {
            let animProp;
            if (SweetAlert.isAnimation(props.closeAnim)) {
                animProp = props.closeAnim;
            }
            else if (SweetAlert.isAnimation(SweetAlert.defaultProps.closeAnim)) {
                animProp = SweetAlert.defaultProps.closeAnim;
            }
            if (animProp) {
                animation = SweetAlert.animationFromProp(animProp);
                hideTimeout(animProp.duration);
                show = true;
            }
        }
        return { show, animation };
    }
    ;
    render() {
        if (!this.state.show) {
            return null;
        }
        return (react_1.default.createElement("div", null,
            this.getInjectedStyles(),
            react_1.default.createElement(Overlay_1.default, { show: !this.props.hideOverlay, onClick: this.onClickOutside, onKeyDown: this.onKeyDown },
                react_1.default.createElement("div", { style: this.getComposedStyle(), tabIndex: 0, onKeyDown: this.onKeyDown, onClick: this.onClickInside, className: 'sweet-alert ' + this.props.customClass },
                    this.getCloseButton(),
                    this.getIcon(),
                    react_1.default.createElement(Title_1.default, null, this.props.title),
                    react_1.default.createElement(Content_1.default, null, this.getAlertContent()),
                    this.getInputField(),
                    this.getValidationMessage(),
                    this.getButtons()))));
    }
}
exports.default = SweetAlert;
SweetAlert.propTypes = prop_types_1.SWEET_ALERT_PROP_TYPES;
SweetAlert.defaultProps = default_props_1.SWEET_ALERT_DEFAULT_PROPS;
SweetAlert.SuccessIcon = SuccessIcon_1.default;
SweetAlert.ErrorIcon = ErrorIcon_1.default;
SweetAlert.InfoIcon = InfoIcon_1.default;
SweetAlert.WarningIcon = WarningIcon_1.default;
SweetAlert.CustomIcon = CustomIcon_1.default;
SweetAlert.Buttons = Buttons_1.default;
SweetAlert.Input = Input_1.default;
SweetAlert.ValidationMessage = ValidationMessage_1.default;
SweetAlert.Title = Title_1.default;
SweetAlert.Content = Content_1.default;
SweetAlert.getStateFromProps = (props) => {
    const type = SweetAlert.getTypeFromProps(props);
    return {
        type,
        focusConfirmBtn: props.focusConfirmBtn && type !== 'input',
        focusCancelBtn: props.focusCancelBtn && type !== 'input',
        dependencies: props.dependencies,
    };
};
SweetAlert.getTypeFromProps = (props) => {
    if (props.type)
        return props.type;
    if (props.secondary)
        return 'secondary';
    if (props.info)
        return 'info';
    if (props.success)
        return 'success';
    if (props.warning)
        return 'warning';
    if (props.danger || props.error)
        return 'danger';
    if (props.input)
        return 'input';
    if (props.custom)
        return 'custom';
    return 'default';
};
//# sourceMappingURL=SweetAlert.js.map